
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : figures-utils.scm
;; DESCRIPTION : utilities routines for the extension of graphics mode
;; COPYRIGHT   : Emmanuël Corcelle (corcelle at gmail dot com)
;;
;; This software falls under the GNU general public license version 3 or later.
;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(texmacs-module (figures-utils)
  (:use (utils library cursor)
	(utils library tree)
	(graphics graphics-utils)))
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                Drawing window
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(tm-define (make-fig-7x5cm-blank)
  (graphics-reset-context 'begin)
  (insert-raw-go-to 
   '(with "gr-mode" "point" 
          "gr-frame" (tuple "scale" "1cm" (tuple "1.3cm" "0.8cm"))
          "gr-geometry" (tuple "geometry" "7cm" "4.5cm")
	(graphics ""))
   '(6 0)))

(tm-define (make-fig-7x5cm-milli)
  (graphics-reset-context 'begin)
  (insert-raw-go-to 
   '(with "gr-mode" "point" 
        "gr-frame" (tuple "scale" "1cm" (tuple "1.3cm" "0.8cm"))
        "gr-geometry" (tuple "geometry" "7cm" "4.5cm")
	"gr-grid" (tuple "cartesian" (point "0" "0") "1")
	"gr-grid-old" (tuple "cartesian" (point "0" "0") "1")
	"gr-edit-grid-aspect" (tuple (tuple "axes" "nones") (tuple "10" "none") (tuple "1" "none"))
	"gr-edit-grid" (tuple "cartesian" (point "0" "0") "1")
	"gr-edit-grid-old" (tuple "cartesian" (point "0" "0") "1")
	(graphics ""))
   '(16 0)))

(tm-define (make-fig-7x5cm-5x5mm)
  (graphics-reset-context 'begin)
  (insert-raw-go-to 
   '(with "gr-mode" "point" 
        "gr-frame" (tuple "scale" "1cm" (tuple "1.3cm" "0.8cm"))
        "gr-geometry" (tuple "geometry" "7cm" "4.5cm")
	"gr-grid" (tuple "cartesian" (point "0" "0") "0.5")
	"gr-grid-old" (tuple "cartesian" (point "0" "0") "0.5")
	"gr-edit-grid-aspect" (tuple (tuple "axes" "nones") (tuple "1" "none") (tuple "1" "none"))
	"gr-edit-grid" (tuple "cartesian" (point "0" "0") "0.5")
	"gr-edit-grid-old" (tuple "cartesian" (point "0" "0") "0.5")
	"gr-grid-aspect-props" (tuple (tuple "axes" "grey") (tuple "1" "#c0c0c0") (tuple "10" "#e0e0ff"))
	"gr-grid-aspect" (tuple (tuple "axes" "grey") (tuple "1" "#c0c0c0"))
	(graphics ""))
   '(20 0)))

(tm-define (make-fig-7x5cm-8x8mm)
  (graphics-reset-context 'begin)
  (insert-raw-go-to 
   '(with "gr-mode" "point" 
        "gr-frame" (tuple "scale" "1cm" (tuple "1.3cm" "0.8cm"))
        "gr-geometry" (tuple "geometry" "7cm" "4.5cm")
	"gr-grid" (tuple "cartesian" (point "0" "0") "0.8")
	"gr-grid-old" (tuple "cartesian" (point "0" "0") "0.8")
	"gr-edit-grid-aspect" (tuple (tuple "axes" "nones") (tuple "1" "none") (tuple "4" "none"))
	"gr-edit-grid" (tuple "cartesian" (point "0" "0") "0.8")
	"gr-edit-grid-old" (tuple "cartesian" (point "0" "0") "0.8")
	"gr-grid-aspect-props" (tuple (tuple "axes" "grey") (tuple "1" "#c0c0c0") (tuple "4" "#e0e0ff"))
	"gr-grid-aspect" (tuple (tuple "axes" "grey") (tuple "1" "#c0c0c0") (tuple "4" "#e0e0ff"))
	(graphics ""))
   '(20 0)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                             Tools
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(tm-define (make-fig-new-middle)   ;;; voir avec (define (move-point) ...) in graphics-edit.scm
  (object_create 'point 3 1))
  
(tm-define (fig-new-middle)
  (graphics-set-mode "arc") 
  (graphics-choose-point)
  (graphics-insert-point x y)
  (make-fig-new-leave))
  
;; fig-division for complexe numbers
(tm-define (fig-/ z1 z2)
  (let* ((a (real-part z1))
         (b (imag-part z1))
         (c (real-part z2))
         (d (imag-part z2))
         (e (/ (+ (* a c) (* b d)) (+ (* c c) (* d d))))
         (f (/ (- (* b c) (* a d)) (+ (* c c) (* d d)))))
  (make-rectangular e f)
  ))

;; insert a new leave in the "graphics" tree
(tm-define (make-fig-new-leave)
  (tree-insert! (tree-ref (cursor-tree) :up) 0 '(""))
  )

;; sum of the numbers of a list ls
(tm-define (fig-sum ls)
  (if (null? ls)
      0
      (+ (car ls) (fig-sum (cdr ls)))))

;; "isobarycenter" of the numbers of a list ls
(tm-define (fig-isobary ls) 
  (fig-/ (fig-sum ls) (length ls)))

;; "middle" of 2 numbers
(tm-define (fig-middle z1 z2) 
  (fig-/ (+ z1 z2) 2))
  
;; define pi
(tm-define (pi) (acos -1))  
  
;; transform an angle from degrees to radians
(tm-define (fig-deg->rad a)
  (/ (* (pi) a) 180))
  
;; transform an angle in [-pi;pi[ to an angle in [0;2pi[   (to use with angle)
(tm-define (fig-posangle a)
  (if (< a 0)
      (+ a (* 2 (pi)))
      a))
  
;; give the rotation of z1 (center z and angle a)
(tm-define (fig-rotation z1 z a)
  (+ z (* (- z1 z) (make-polar 1 a))))
  
;; give the projection of z on line l perpendicularly to line l
(tm-define (fig-projection z ls-pts)
  (let* ((a (list-ref ls-pts 0))
         (b (list-ref ls-pts 1))
         (c (+ z (fig-normal (- a b)))))
  (fig-inter-lines a b c z)
  ))

;; give the reflection of z in respect of line '(a b)
(tm-define (fig-line-reflection z ls-pts)
  (let* ((m (fig-projection z ls-pts)))
  (+ z (* 2 (- m z)))
  ))

;; unit vector of z
(tm-define (fig-unit z)
  (fig-/ z (magnitude z)))
  
;; normal vector of z
(tm-define (fig-normal z)
  (fig-unit (* z (make-polar 1 (/ (pi) 2)))))
  
;; give the list of coeff '(a b c) of the equation of line (AB) (ax+by=c) knowing zA and zB
(tm-define (fig-equation-line z1 z2)
  (let ((x1 (real-part z1))
        (y1 (imag-part z1))
        (x2 (real-part z2))
        (y2 (imag-part z2)))
  (if (!= z1 z2)
      `(,(- y2 y1) ,(- x1 x2) ,(- (* x1 y2) (* x2 y1))))))

;; give the complex intersection of two lines of equations a1x+b1y=c1 '(a1 b1 c1) and a2x+b2y=c2 '(a2 b2 c2)
(tm-define (fig-inter-lines-list ls1 ls2)
  (let* ((a1 (list-ref ls1 0))
         (b1 (list-ref ls1 1))
         (c1 (list-ref ls1 2))
         (a2 (list-ref ls2 0))
         (b2 (list-ref ls2 1))
         (c2 (list-ref ls2 2))
         (x (/ (- (* b2 c1) (* b1 c2)) (- (* a1 b2) (* a2 b1))))
         (y (/ (- (* a1 c2) (* a2 c1)) (- (* a1 b2) (* a2 b1)))))
  (if (!= 0 (- (* a1 b2) (* a2 b1)))
      (make-rectangular x y))))
  
;; give the complex intersection (with positive imaginary part) of the circle of center 0 and radius r1
;; and of the circle of center a and radius r2     ;;;; to adapt for other cases
(tm-define (fig-inter-circles ls-circ1 ls-circ2)
  (let* ((z1 (list-ref ls-circ1 0))
         (r1 (list-ref ls-circ1 1))
         (z2 (list-ref ls-circ2 0))
         (r2 (list-ref ls-circ2 1))
         (x (+ (fig-/ z2 2) (fig-/ (- (* r1 r1) (* r2 r2)) (* 2 z2))))
         (y (sqrt (- (* r1 r1) (* x x)))))
  (make-rectangular x y)
  ))
      
;; give a complex point (at length len of z2) on the bisector of an angle defined by z1 z2 z3
(tm-define (fig-point-on-bisector z1 z2 z3 len)
  (+ z2 (* len (fig-unit (- (fig-rotation z1 z2 (fig-/ (fig-posangle (angle (fig-/ (- z3 z2) (- z1 z2)))) 2)) z2)))))

;; circular permutation of list ls of 1 position right
(tm-define (fig-list-permut+1 ls)
  (append (list-tail ls 1) (list (list-ref ls 0))))

;; circular permutation of list ls of 1 position left
(tm-define (fig-list-permut-1 ls)
  (let ((n (length ls)))
  (append (list (list-ref ls (- n 1))) (reverse (list-tail (reverse ls) 1)))))

;; give the list of the middles of the numbers of list ls
(tm-define (fig-list-middles ls)
  (map fig-middle ls (fig-list-permut+1 ls)))

;; transform a complex z into (point "x" "y")
(tm-define (fig-complex->point z)	
	`(point ,(number->string (real-part z)) ,(number->string (imag-part z))))

;; transform a complex z into : '(x y)    ;; not used for the moment
(tm-define (fig-complex->coord z)	
	`(,(number->string (real-part z)) ,(number->string (imag-part z))))

;; transform a list of complexes zi into a list (... (point "xi" "yi") ...) 
(tm-define (fig-insert-pts ls-pt)
  (map (lambda (z) (fig-complex->point z)) ls-pt))

;; give the length of the third side of a right triangle
(tm-define (fig-pythagore ls-len type)
  (let ((a (list-ref ls-len 0))
        (b (list-ref ls-len 1)))
  (if (string=? type "hyp")
      (+ (* a a) (* b b))
      (- (* (max a b) (max a b)) (* (min a b) (min a b))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                            Points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; give the complex intersection of 2 lines (a b) and (c d) TODO :  adapt with '(a b) and '(c d)
(tm-define (fig-inter-lines a b c d)
  (fig-inter-lines-list (fig-equation-line a b) (fig-equation-line c d)))

;; insert the intersection point of 2 lines (a b) and (c d)
(tm-define (make-fig-inter-lines ls1 ls2)
  (let* ((a (list-ref ls1 0))
         (b (list-ref ls1 1))
         (c (list-ref ls2 0))
         (d (list-ref ls2 1)))
  (if (!= (angle (- a b)) (angle (- c d)))
      (begin
	(graphics-set-mode "point") 
	(insert (fig-complex->point (fig-inter-lines a b c d)))
	(make-fig-new-leave)))))

;; insert a cross ( x ) at the point z   TODO : add the label of the point if wanted
(tm-define (make-fig-cross z size)
  (graphics-set-mode "point") 
  (insert `(gr-group (line ,@(fig-insert-pts `(,(+ z (* size 1+i)) ,(- z (* size 1+i))))) 
		     (line ,@(fig-insert-pts `(,(+ z (* size 1-i)) ,(- z (* size 1-i)))))))
  (make-fig-new-leave))

;; insert a cross ( x ) at the point z   TODO : add the label of the point if wanted
(tm-define (make-fig-cross2)
  (graphics-set-mode "point") 
  (insert `(gr-group (line ,@(fig-insert-pts `(,(+ z (* size 1+i)) ,(- z (* size 1+i))))) 
		     (line ,@(fig-insert-pts `(,(+ z (* size 1-i)) ,(- z (* size 1-i)))))))
  (make-fig-new-leave))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                        Lines and polygons
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; insert a line (list of complexes)
(tm-define (make-fig-line ls-pt)
  (graphics-set-mode "line") 
  (insert `(line ,@(fig-insert-pts ls-pt)))
  (make-fig-new-leave))
  
;; insert a segment (list of 2 complex) with the labels of his summits (list of strings) and sides (list of strings)
(tm-define (make-fig-seg-lab ls-pts ls-lab ls-len ext)
  (let* ((z1 (list-ref ls-pts 0))
         (z2 (list-ref ls-pts 1))
         (z0 (+ z1 (fig-normal (- z1 z2))))
         (z3 (+ z2 (fig-normal (- z1 z2)))))
  (make-fig-line ls-pts)
  (if (string=? ext "extremites")
    (begin
      (make-fig-perpendicular ls-pts z1 "none" 0.1 0.1)
      (make-fig-perpendicular ls-pts z2 "none" 0.1 0.1)))
  (make-fig-labels-sides ls-pts ls-len 0.5)
  (make-fig-label (+ z1 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 0))
  (make-fig-label (+ z2 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 1))))
  
;; insert a line (droite - list of 2 complex) with the labels of his summits (list of strings) and sides (list of strings)
(tm-define (make-fig-droite-lab ls-pts ls-lab mark size1 size2)
  (let* ((z1 (list-ref ls-pts 0))
         (z2 (list-ref ls-pts 1))
         (z3 (+ z2 (* size2 (fig-unit (- z2 z1)))))
         (z4 (+ z1 (* size1 (fig-unit (- z1 z2))))))
  (make-fig-line `(,z3 ,z4))
  (if (string=? mark "points")
    (begin
      (make-fig-perpendicular ls-pts z1 "none" 0.1 0.1)
      (make-fig-perpendicular ls-pts z2 "none" 0.1 0.1)
      (make-fig-label (+ z1 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 0))
      (make-fig-label (+ z2 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 1))))))
      
;; insert a line (droite) of equation : ax+by+c=0  or  y=ax+b 
(tm-define (make-fig-droite-equa ls-coef size1 size2)
  (if (= (length ls-coef) 3)
      (begin
        (let* ((a (list-ref ls-coef 0))
               (b (list-ref ls-coef 1))
               (c (list-ref ls-coef 2))
               (z1 (if (= b 0)
                       (if (!= c 0)
                           (make-rectangular (/ (- 0 c) a) 1))
                       (if (= c 0)
                           (make-rectangular 1 (/ (- 0 a) b))
                           (make-rectangular 0 (/ (- 0 c) b)))))
               (z2 (if (= a 0)
                       (make-rectangular 1 (/ (- 0 c) b))
                       (make-rectangular (/ (- 0 c) a) 0))))
        (make-fig-droite-lab `(,z1 ,z2) '() "no-points" size1 size2)))
      (begin
        (let* ((a (list-ref ls-coef 0))
               (b (list-ref ls-coef 1))
               (z1 (make-rectangular 0 b))
               (z2 (make-rectangular 1 (+ (* a 1) b))))
        (make-fig-droite-lab `(,z1 ,z2) '() "no-points" size1 size2)))
  ))

;; insert a polygon (list of complexes)
(tm-define (make-fig-polygon ls-pt)
  (graphics-set-mode "cline") 
  (insert `(cline ,@(fig-insert-pts ls-pt)))
  (make-fig-new-leave))

;; give perpendicular line to line (z1 z2) containing point z (and of length size1 + size2)  TODO :  adapt with '(z1 z2)
(tm-define (fig-perpendicular z1 z2 z size1 size2)
  `(line ,@(fig-insert-pts `(,(+ z (* size1 (fig-normal (- z1 z2)))) ,(+ z (* size2 (fig-normal (- z2 z1))))))))

;; insert perpendicular line to line (z1 z2) containing point z (and of length size1 + size2) 
(tm-define (make-fig-perpendicular ls-pts z mark size1 size2)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1))
  	 (zinter (fig-inter-lines z1 z2 z (+ z (fig-normal (- z1 z2))))))
  (graphics-set-mode "line") 
  (insert (fig-perpendicular z1 z2 z size1 size2))
  (make-fig-new-leave)
  (if (string=? mark "right")
      (if (= z z1) 
      	(make-fig-mark-right `(,z2 ,zinter ,(+ zinter (fig-normal (- z1 z2)))) 0.3)
      	(make-fig-mark-right `(,z1 ,zinter ,(+ zinter (fig-normal (- z1 z2)))) 0.3))
      (make-fig-new-leave))))

;; give parallel line to line (z1 z2) containing point z (and of length size1 + size2)
(tm-define (fig-parallel z1 z2 z size1 size2)
  `(line ,@(fig-insert-pts `(,(+ z (* size1 (fig-unit (- z1 z2)))) ,(+ z (* size2 (fig-unit (- z2 z1))))))))

;; insert parallel line to line (z1 z2) containing point z (and of length size1 + size2)  
(tm-define (make-fig-parallel ls-pts z size1 size2)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1)))
  (graphics-set-mode "line") 
  (insert (fig-parallel z1 z2 z size1 size2))
  (make-fig-new-leave)))
  
;; give the bisector line of an angle defined by 3 points z1 z2 z3
(tm-define (fig-bisector z1 z2 z3 size1 size2)
  (let ((z (fig-point-on-bisector z1 z2 z3 1)))
    `(line ,@(fig-insert-pts `(,(+ z2 (* size1 (fig-unit (- z z2)))) ,(+ z2 (* size2 (fig-unit (- z2 z)))))))))

;; insert the bisector line of an angle defined by 3 points z1 z2 z3
(tm-define (make-fig-bisector ls-pts mark sizemark size1 size2)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1))
  	 (z3 (list-ref ls-pts 2))
  	 (z (fig-point-on-bisector z1 z2 z3 1)))
    (graphics-set-mode "line") 
    (insert (fig-bisector z1 z2 z3 size1 size2))
    (make-fig-new-leave)
    (make-fig-mark-angle `(,z1 ,z2 ,z) mark sizemark)
    (make-fig-mark-angle `(,z ,z2 ,z3) mark (+ sizemark 0.05))))
    
;; insert the perpendicular bisector of a segment defined by 2 points z1 z2
(tm-define (make-fig-mediatrice ls-pts mark size1 size2)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1))
  	 (m (fig-middle z1 z2)))  
  (make-fig-perpendicular ls-pts m "right" size1 size2)
  (make-fig-mark-seg `(,z1 ,m) mark 0.15 +i)
  (make-fig-mark-seg `(,z2 ,m) mark 0.15 +i)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                            Circles
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
;; insert a circle of center z_center and radius r
(tm-define (make-fig-circle-rad z_center r mark lab_center)
  (let* ((z1 (+ z_center r))
         (z2 (fig-rotation z1 z_center (/ (pi) 2)))
         (z3 (fig-rotation z1 z_center (pi))))
  (if (string=? mark "center")
      (begin
        (make-fig-cross z_center 0.1)
        (graphics-set-mode "text-at")
        (insert `(with "text-at-halign" "center" "text-at-valign" "center" (text-at (math ,lab_center) ,(fig-complex->point (- z_center 0.3+0.3i)))))
        (make-fig-new-leave)))
  (graphics-set-mode "carc") 
  (insert `(carc ,@(fig-insert-pts `(,z1 ,z2 ,z3))))
  (make-fig-new-leave)))
  
;; insert a circle of center z_center    TODO : add point and label if wanted
(tm-define (make-fig-circle-pt z_center z_pt mark lab_center)
  (make-fig-circle-rad z_center (magnitude (- z_center z_pt)) mark lab_center))

;; insert an arc of circle of center z_center from point z1 to point z2
(tm-define (make-fig-arc-pt ls-pts mark lab_center)
  (let* ((z1 (list-ref ls-pts 0))
         (z_center (list-ref ls-pts 1))
         (z2 (list-ref ls-pts 2))
         (r (magnitude (- z1 z_center)))
         (z_mid (fig-point-on-bisector z1 z_center z2 r))
         (z_end (+ z_center (* r (fig-unit (- z2 z_center))))))
  (if (string=? mark "center")
      (begin
        (make-fig-cross z_center 0.1)
        (graphics-set-mode "text-at")
        (insert `(with "text-at-halign" "center" "text-at-valign" "center" (text-at (math ,lab_center) ,(fig-complex->point (- z_center 0.3+0.3i)))))
        (make-fig-new-leave)))
  (graphics-set-mode "arc") 
  (insert `(arc ,@(fig-insert-pts `(,z1 ,z_mid ,z_end))))
  (make-fig-new-leave)))

;; insert an arc of circle of center z_center from angle a1 to angle a2 (in degrees)
(tm-define (make-fig-arc-rad z_center rad angles mark lab_center)
  (let* ((z1 (+ z_center (make-polar rad (fig-deg->rad (list-ref angles 0)))))
         (z2 (+ z_center (make-polar rad (fig-deg->rad (list-ref angles 1))))))
  (make-fig-arc-pt `(,z1 ,z_center ,z2) mark lab_center)))

;; insert an arc of circle of center z_center from point z1 to point z2 with choosen style
(tm-define (make-fig-arc-pt-style ls-pts mark lab_center)
  (let* ((z1 (list-ref ls-pts 0))
         (z_center (list-ref ls-pts 1))
         (z2 (list-ref ls-pts 2))
         (r (magnitude (- z1 z_center)))
         (z_mid (fig-point-on-bisector z1 z_center z2 r))
         (z_end (+ z_center (* r (fig-unit (- z2 z_center))))))
  (if (string=? mark "center")
      (begin
        (make-fig-cross z_center 0.1)
        (make-fig-new-leave)))
  (graphics-set-mode "arc")
  (insert `(with "dash-style" (tuple "1" "1" "1" "1" "0" "1" "0") "line-arrows" (tuple (with "dash-style" "none" (line (tuple "-10ln" "6ln") (tuple "0ln" "0ln") (tuple "-10ln" "-6ln")))) (arc ,@(fig-insert-pts `(,z1 ,z_mid ,z_end)))))
  (make-fig-new-leave)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                            Marks
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; insert a right angle defined by 3 points and his size
(tm-define (make-fig-mark-right ls-pts size)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1))
  	 (z3 (list-ref ls-pts 2))
  	 (a (+ z2 (* size (fig-unit (- z1 z2)))))
         (c (+ z2 (* size (fig-unit (- z3 z2)))))
         (b (+ z2 (+ (- a z2) (- c z2)))))
    (graphics-set-mode "line") 
    (insert `(line ,@(fig-insert-pts `(,a ,b ,c))))
    (make-fig-new-leave)))

;; give an angle (arc) defined by 3 points and his size
(tm-define (fig-angle z1 z2 z3 size)
  (let* ((a (+ z2 (* size (fig-unit (- z1 z2)))))
         (c (+ z2 (* size (fig-unit (- z3 z2)))))
         (b (fig-point-on-bisector z1 z2 z3 size))) 
    `(arc ,@(fig-insert-pts `(,a ,b ,c)))))
    
;; insert a marked angle defined by 3 points, his type and his size
(tm-define (make-fig-mark-angle ls-pts type size)  
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1))
  	 (z3 (list-ref ls-pts 2)))
  (graphics-set-mode "arc") 
  (cond ((string=? type "simple")
	   (insert (fig-angle z1 z2 z3 size))
	   (make-fig-new-leave))
         ((string=? type "double") 
           (insert `(gr-group 	,(fig-angle z1 z2 z3 size)
           			,(fig-angle z1 z2 z3 (+ size 0.1))))
           (make-fig-new-leave))
         ((string=? type "triple")  
           (insert `(gr-group 	,(fig-angle z1 z2 z3 size)
	   			,(fig-angle z1 z2 z3 (+ size 0.1))
           			,(fig-angle z1 z2 z3 (+ size 0.2))))
           (make-fig-new-leave))
         ((string=? type "circle")  
           (insert `(gr-group 	,(fig-angle z1 z2 z3 size)
           			(with "point-style" "round" ,(fig-complex->point (fig-point-on-bisector z1 z2 z3 size)))))
           (make-fig-new-leave))))) 

;; give a mark (line) on a segment z1 z2 at a distance dist from the middle
(tm-define (fig-mark z1 z2 dist size orient)
  (let* ((m (+ (fig-middle z1 z2) (* dist (fig-unit (- z1 z2)))))
         (m1 (+ m (* (* size (fig-unit orient)) (fig-unit (- z1 z2)))))
         (m2 (+ m (* (* size (- 0 (fig-unit orient))) (fig-unit (- z1 z2))))))
    `(line ,@(fig-insert-pts `(,m1 ,m2)))))

;; insert a mark on a segment z1 z2
(tm-define (make-fig-mark-seg ls-pts type size orient)
  (let* ((z1 (list-ref ls-pts 0))
  	 (z2 (list-ref ls-pts 1)))
  (graphics-set-mode "line") 
  (cond ((string=? type "simple") 
	   (insert (fig-mark z1 z2 0 size orient))
 	   (make-fig-new-leave))
         ((string=? type "double") 
           (insert `(gr-group 	,(fig-mark z1 z2 0.05 size orient)
           			,(fig-mark z1 z2 -0.05 size orient)))
 	   (make-fig-new-leave))
         ((string=? type "triple") 
           (insert `(gr-group 	,(fig-mark z1 z2 0.1 size orient)
	   			,(fig-mark z1 z2 0 size orient)
           			,(fig-mark z1 z2 -0.1 size orient)))
 	   (make-fig-new-leave))
         ((string=? type "cross")  
           (insert `(gr-group 	,(fig-mark z1 z2 0 size (+ 0.5 orient))
           			,(fig-mark z1 z2 0 size (+ -0.5 orient))))
 	   (make-fig-new-leave))
         ((string=? type "circle") 
	   (insert `(with "point-style" "round" ,(fig-complex->point (fig-middle z1 z2))))
   	   (make-fig-new-leave)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                          Transformations            
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; give the translated list (of vector z_vector) of a list of complexes
(tm-define (fig-ls-translation ls-pts z_vector)
  (map (lambda (z) (+ z z_vector)) ls-pts))
  
;; insert an object (polygon, line) and the translation (of vector z-vector) of this object
(tm-define (make-fig-translation type ls-pts ls-lab z_vector constr)
  (let ((ls-pts2 (fig-ls-translation ls-pts z_vector))
        (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab)))     ;; TODO replace with : (rprime "'")
  (if (string=? constr "construction")
      (begin
        (graphics-set-dash-style "1111010")
        (map (lambda (x y) (graphics-set-mode "line") 
                           (insert `(with "dash-style" (tuple "1" "1" "1" "1" "0" "1" "0") "line-arrows" (tuple (with "dash-style" "none" (line (tuple "-10ln" "6ln") (tuple "0ln" "0ln") (tuple "-10ln" "-6ln")))) (line ,@(fig-insert-pts `(,x ,y)))))
                           (make-fig-new-leave)) 
             ls-pts ls-pts2)))
  (cond ((string=? type "polygon")			;; (concat "Some " (with "color" "blue" "blue") " text.")
  	   (make-fig-polygon-lab ls-pts ls-lab '())
  	   (make-fig-polygon-lab ls-pts2 ls-lab2 '()))
  	((string=? type "line")
  	   (make-fig-line-lab ls-pts ls-lab '())
  	   (make-fig-line-lab ls-pts2 ls-lab2 '()))
  	((string=? type "segment")
  	   (make-fig-seg-lab ls-pts ls-lab `() "extremites")
  	   (make-fig-seg-lab ls-pts2 ls-lab2 `() "extremites"))
  	((string=? type "point")
  	   (let* ((z1 (list-ref ls-pts 0)) 
  	          (zp1 (list-ref ls-pts2 0)))
  	   (make-fig-cross z1 0.1)
  	   (make-fig-cross zp1 0.1)
           (make-fig-label (+ z1 (* 0.4 (fig-normal (- z1 zp1)))) (list-ref ls-lab 0))
           (make-fig-label (+ zp1 (* 0.4 (fig-normal (- z1 zp1)))) (list-ref ls-lab2 0))))
  	((string=? type "circle")
  	   (let* ((z1 (list-ref ls-pts 0))
  	          (z2 (list-ref ls-pts 1)) 
  	          (zp1 (list-ref ls-pts2 0))
  	          (zp2 (list-ref ls-pts2 1)))
  	   (make-fig-cross z2 0.1)
  	   (make-fig-cross zp2 0.1)
           (make-fig-label (+ z2 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab 1))
           (make-fig-label (+ zp2 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab2 1))
  	   (make-fig-circle-pt z1 z2 "center" (list-ref ls-lab 0))
  	   (make-fig-circle-pt zp1 zp2 "center" (list-ref ls-lab2 0)))))	 
  (graphics-set-mode "text-at")
  (graphics-set-dash-style "default")
  ))

;; give the rotated list (of center z_center and angle a in degrees) of a list of complexes
(tm-define (fig-ls-rotation ls-pts z_center a)
  (map (lambda (z) (fig-rotation z z_center a)) ls-pts))

;; insert an object (polygon, line) and the rotation (of center z-center and angle a in degrees) of this object
(tm-define (make-fig-rotation type ls-pts ls-lab z_center lab_center a constr)
  (let ((ls-pts2 (fig-ls-rotation ls-pts z_center (fig-deg->rad a)))
        (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab)))
  ; construction lines (arcs)          
  (if (string=? constr "construction")
      (begin
        (map (lambda (x y) 
               (graphics-set-mode "line") 
               (graphics-set-dash-style "1111010")
               (make-fig-arc-pt-style `(,x ,z_center ,y) "no-center" "")) 
               ls-pts ls-pts2)))
  ; mark and label of center
  (make-fig-cross z_center 0.1)
  (graphics-set-mode "text-at")
  (insert `(with "text-at-halign" "center" "text-at-valign" "center" (text-at (math ,lab_center) ,(fig-complex->point (- z_center 0.3+0.3i)))))
  (make-fig-new-leave)    
  ; object and rotated object                       
  (cond ((string=? type "polygon")
  	   (make-fig-polygon-lab ls-pts ls-lab '())
  	   (make-fig-polygon-lab ls-pts2 ls-lab2 '()))
  	((string=? type "line")
  	   (make-fig-line-lab ls-pts ls-lab '())
  	   (make-fig-line-lab ls-pts2 ls-lab2 '()))
  	((string=? type "segment")
  	   (make-fig-seg-lab ls-pts ls-lab `() "extremites")
  	   (make-fig-seg-lab ls-pts2 ls-lab2 `() "extremites"))
  	((string=? type "point")
  	   (let* ((z1 (list-ref ls-pts 0)) 
  	          (z2 (list-ref ls-pts2 0)))
  	   (make-fig-cross (list-ref ls-pts 0) 0.1)
  	   (make-fig-cross (list-ref ls-pts2 0) 0.1)
           (make-fig-label (+ z1 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 0))
           (make-fig-label (+ z2 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab2 0))))
  	((string=? type "circle")
  	   (let* ((z1 (list-ref ls-pts 0))
  	          (z2 (list-ref ls-pts 1)) 
  	          (zp1 (list-ref ls-pts2 0))
  	          (zp2 (list-ref ls-pts2 1)))
  	   (make-fig-cross z2 0.1)
  	   (make-fig-cross zp2 0.1)
           (make-fig-label (+ z2 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab 1))
           (make-fig-label (+ zp2 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab2 1))
  	   (make-fig-circle-pt z1 z2 "center" (list-ref ls-lab 0))
  	   (make-fig-circle-pt zp1 zp2 "center" (list-ref ls-lab2 0)))))
  (graphics-set-mode "text-at")
  (graphics-set-dash-style "default")
  ))

;; insert an object (polygon, line) and the point reflection (of center z-center) of this object
(tm-define (make-fig-pt-reflection type ls-pts ls-lab z_center lab_center constr)
  (let ((ls-pts2 (fig-ls-rotation ls-pts z_center (fig-deg->rad 180))))
  (if (string=? constr "construction")
      (begin
        (graphics-set-dash-style "1111010")
        (map (lambda (x y) (graphics-set-mode "line") 
                           (insert `(with "dash-style" (tuple "1" "1" "1" "1" "0" "1" "0") (line ,@(fig-insert-pts `(,x ,y)))))
                           (make-fig-new-leave)) 
             ls-pts ls-pts2)))	 
  (graphics-set-mode "text-at")
  (graphics-set-dash-style "default"))
  (make-fig-rotation type ls-pts ls-lab z_center lab_center 180 "noconstruction"))
  
;; give the reflected list (in respect of line '(a b)) of a list of complexes
(tm-define (fig-ls-line-reflection ls-pts ls-axe)
  (map (lambda (z) (fig-line-reflection z ls-axe)) ls-pts))

;; insert an object (polygon, line) and the reflection (in respect of line ls-axe) of this object
(tm-define (make-fig-line-reflection type ls-pts ls-lab ls-axe ls-lab-axe mark constr size)
  (let ((ls-pts2 (reverse (fig-ls-line-reflection ls-pts ls-axe)))
        (ls-lab2 (reverse (map (lambda (str) (string-append str "'")) ls-lab))))
  (if (string=? constr "construction")
      (begin
        (graphics-set-dash-style "1111010")
        (map (lambda (x y) (graphics-set-mode "line") 
                           (insert `(with "dash-style" (tuple "1" "1" "1" "1" "0" "1" "0") (line ,@(fig-insert-pts `(,x ,y)))))
                           (make-fig-new-leave)) 
             ls-pts (reverse ls-pts2))))
  (cond ((string=? type "polygon")
  	   (make-fig-polygon-lab ls-pts ls-lab '())
  	   (make-fig-polygon-lab ls-pts2 ls-lab2 '()))
  	((string=? type "line")
  	   (make-fig-line-lab ls-pts ls-lab '())
  	   (make-fig-line-lab ls-pts2 ls-lab2 '()))
  	((string=? type "segment")
  	   (make-fig-seg-lab ls-pts ls-lab `() "extremites")
  	   (make-fig-seg-lab ls-pts2 ls-lab2 `() "extremites"))
  	((string=? type "point")
  	   (let* ((z1 (list-ref ls-pts 0)) 
  	          (z2 (list-ref ls-pts2 0)))
  	   (make-fig-cross (list-ref ls-pts 0) 0.1)
  	   (make-fig-cross (list-ref ls-pts2 0) 0.1)
           (make-fig-label (+ z1 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab 0))
           (make-fig-label (+ z2 (* 0.4 (fig-normal (- z1 z2)))) (list-ref ls-lab2 0))))
  	((string=? type "circle")
  	   (let* ((z1 (list-ref ls-pts 0))
  	          (z2 (list-ref ls-pts 1)) 
  	          (zp1 (list-ref ls-pts2 0))
  	          (zp2 (list-ref ls-pts2 1)))
  	   (make-fig-cross z2 0.1)
  	   (make-fig-cross zp1 0.1)
           (make-fig-label (+ z2 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab 1))
           (make-fig-label (+ zp1 (* 0.4 (fig-normal (- z2 zp2)))) (list-ref ls-lab2 0))
  	   (make-fig-circle-pt z1 z2 "center" (list-ref ls-lab 0))
  	   (make-fig-circle-pt zp2 zp1 "center" (list-ref ls-lab2 1)))))
  (make-fig-droite-lab ls-axe ls-lab-axe mark size size)
;  (graphics-set-mode "text-at")
;  (insert `(with "text-at-halign" "center" "text-at-valign" "center" (text-at (math ,lab_center) ,(fig-complex->point (- z_center 0.3+0.3i)))))
;  (make-fig-new-leave)	 
  (graphics-set-mode "text-at")
  (graphics-set-dash-style "default")
  ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                            Labels 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
;; give the complex position of the label of a segment (right side)
(tm-define (fig-label-segment z1 z2 dist)
  (fig-point-on-bisector z1 (fig-middle z1 z2) z2 dist))

;; give the complex position of the label of the k-the summit of a polygon (list of complexes)
;(tm-define (fig-label-summit ls-pt k dist)  
;  (let ((z (list-ref ls-pt k)))
;    (+ z (* dist (fig-/ (- z (fig-isobary ls-pt)) (magnitude (- z (fig-isobary ls-pt))))))))

;; insert a label str at position z
(tm-define (make-fig-label z str)	 
  (graphics-set-mode "text-at")
  (insert `(with "text-at-halign" "center" "text-at-valign" "center" (text-at (math ,str) ,(fig-complex->point z))))
  (make-fig-new-leave))

;; insert the labels (list of strings) of the sides of a polygon (list of complexes)
(tm-define (make-fig-labels-sides ls-pt ls-lab dist)
  (let ((k 0))
    (for-each 
      (lambda (str) 
	(make-fig-label (fig-label-segment (list-ref ls-pt k) (list-ref (fig-list-permut+1 ls-pt) k) dist) str)
	(set! k (+ k 1))) 
      ls-lab)))

;; insert the labels (list of strings) of the summits of a polygon (list of complexes)
(tm-define (make-fig-labels-poly ls-pt ls-lab dist)  
  (let ((k 0))
    (for-each 
      (lambda (str) 
	(make-fig-label (fig-point-on-bisector  (list-ref (fig-list-permut-1 ls-pt) k)
						(list-ref ls-pt k) 
						(list-ref (fig-list-permut+1 ls-pt) k) dist) str)
	(set! k (+ k 1))) 
      ls-lab)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                      Complete polygons
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; insert a line (list of complex points) with the labels of his summits (list of strings) and sides (list of strings)
(tm-define (make-fig-line-lab ls-pt ls-lab ls-length)
  (make-fig-line ls-pt)
  (make-fig-labels-poly ls-pt ls-lab 0.3)
  (make-fig-labels-sides ls-pt ls-length 0.5))

;; insert a polygon (list of complex points) with the labels of his summits (list of strings) and sides (list of strings)
(tm-define (make-fig-polygon-lab ls-pt ls-lab ls-length)
  (make-fig-polygon ls-pt)
  (make-fig-labels-poly ls-pt ls-lab 0.3)
  (make-fig-labels-sides ls-pt ls-length 0.5))

;; give the list of summits of a triangle knowing his lengths
(tm-define (fig-tri-lengths->summits ls-len)
  (let* ((transl 0)       ; TODO : add the translation (and rotation angle) of the summits as parameters 
         (a (list-ref ls-len 0))			;
  	 (b (list-ref ls-len 1))			; lengths of the sides
  	 (c (list-ref ls-len 2))			;
  	 (z1 transl)
         (z2 (+ transl a))
         (z3 (+ transl (fig-inter-circles `(,z1 ,c) `(,z2 ,b)))))
  `(,z1 ,z2 ,z3)
  ))
  
;; insert a triangle with the labels of his summits and if wanted the lengths of the sides 
;; knowing theses lengths of the 3 sides
(tm-define (make-fig-triangle ls-len ls-lab unit mark-lab mark-seg mark-angle)	
  (let* ((a (list-ref ls-len 0))			;
  	 (b (list-ref ls-len 1))			; lengths of the sides
  	 (c (list-ref ls-len 2))			;
  	 (z1 (list-ref (fig-tri-lengths->summits ls-len) 0))		;
  	 (z2 (list-ref (fig-tri-lengths->summits ls-len) 1))		; affixes of the summits
  	 (z3 (list-ref (fig-tri-lengths->summits ls-len) 2))		;
  	 (ls-str (map (lambda (x) (string-append x unit)) (map number->string ls-len))))
  (make-fig-polygon `(,z1 ,z2 ,z3))
  (make-fig-labels-poly `(,z1 ,z2 ,z3) ls-lab 0.3)
  (if (string=? mark-lab "lengths")
    (make-fig-labels-sides `(,z1 ,z2 ,z3) ls-str 0.5))
  ; mark angles and segments
  (if (= b c)
    (begin
      (make-fig-mark-angle `(,z2 ,z1 ,z3) mark-angle 0.4)
      (make-fig-mark-angle `(,z3 ,z2 ,z1) mark-angle 0.4)
      (make-fig-mark-seg `(,z2 ,z3) mark-seg 0.15 +i)
      (make-fig-mark-seg `(,z3 ,z1) mark-seg 0.15 +i)
      (if (= b a)
       (begin
         (make-fig-mark-angle `(,z1 ,z3 ,z2) mark-angle 0.4)
         (make-fig-mark-seg `(,z2 ,z1) mark-seg 0.15 +i))))
    (if (= c a)
        (begin
          (make-fig-mark-angle `(,z3 ,z2 ,z1) mark-angle 0.4)
          (make-fig-mark-angle `(,z1 ,z3 ,z2) mark-angle 0.4)
          (make-fig-mark-seg `(,z1 ,z2) mark-seg 0.15 +i)
          (make-fig-mark-seg `(,z1 ,z3) mark-seg 0.15 +i))
        (if (= b a)
            (begin
              (make-fig-mark-angle `(,z2 ,z1 ,z3) mark-angle 0.4)
              (make-fig-mark-angle `(,z1 ,z3 ,z2) mark-angle 0.4)
              (make-fig-mark-seg `(,z1 ,z2) mark-seg 0.15 +i)
              (make-fig-mark-seg `(,z3 ,z2) mark-seg 0.15 +i)))))
  )) 
  
; insert a right triangle knowing the lengths of 2 sides
(tm-define (make-fig-right-triangle ls-len ls-lab unit mark-lab mark-seg mark-angle)
  (let* ((a1 (list-ref ls-len 0))	;
  	 (b1 (list-ref ls-len 1))	; adjacent sides
  	 (c1 (list-ref ls-len 2))		; hypotenuse
  	 (miss-side-type (if (= c1 0)
  	                    "hyp"
  	                    "adj"))
  	 (a (if (= a1 0)
  	        (sqrt (fig-pythagore `(,b1 ,c1) "adj"))
  	         a1))
  	 (b (if (= b1 0)
  	        (sqrt (fig-pythagore `(,a1 ,c1) "adj"))
  	         b1))
  	 (c (if (= c1 0)
  	        (sqrt (fig-pythagore `(,a1 ,b1) "hyp"))
  	         c1)))
  (make-fig-triangle `(,a ,c ,b) ls-lab unit mark-lab mark-seg mark-angle))
  (make-fig-mark-right '(1 0 +i) 0.3))
  
;; insert an isosceles triangle knowing the lengths of 2 sides
(tm-define (make-fig-isosceles ls-len ls-lab unit mark-lab mark-seg mark-angle)
  (make-fig-triangle ls-len ls-lab unit mark-lab mark-seg mark-angle))
  
;; insert an equilateral triangle knowing the lengths of 1 side
(tm-define (make-fig-equilateral ls-len ls-lab unit mark-lab mark-seg mark-angle)
  (make-fig-triangle `(,(list-ref ls-len 0) ,(list-ref ls-len 0) ,(list-ref ls-len 0)) ls-lab unit mark-lab mark-seg mark-angle))

;; insert a rectangle with the labels of his summits (list of strings) and sides (list of 2 strings)
;; knowing theses lengths of the sides
(tm-define (make-fig-rectangle-lab ls-len ls-lab unit mark-lab mark-seg-L mark-seg-l)
  (let* ((L (list-ref ls-len 0))
         (l (list-ref ls-len 1))
         (z1 0)
  	 (z2 L)
  	 (z3 (+ L (* l +i)))
  	 (z4 (* l +i)))
  (make-fig-polygon `(,z1 ,z2 ,z3 ,z4))
  (make-fig-mark-right '(1 0 +i) 0.3)
  (make-fig-labels-poly `(,z1 ,z2 ,z3 ,z4) ls-lab 0.3)
  (if (string=? mark-lab "lengths")
  (make-fig-labels-sides `(,z1 ,z2 ,z3) ls-str 0.5))
  (make-fig-mark-seg `(,z1 ,z2) mark-seg-L 0.15 +i)
  (make-fig-mark-seg `(,z2 ,z3) mark-seg-l 0.15 +i)
  (make-fig-mark-seg `(,z3 ,z4) mark-seg-L 0.15 +i)
  (make-fig-mark-seg `(,z4 ,z1) mark-seg-l 0.15 +i)
  ))

;; insert a square with the labels of his summits (list of strings) and sides (list of 1 string)
;; knowing theses lengths of the sides
(tm-define (make-fig-square-lab ls-len ls-lab unit mark-lab mark-seg)
  (make-fig-rectangle-lab `(,(list-ref ls-len 0) ,(list-ref ls-len 0)) ls-lab unit mark-lab mark-seg mark-seg))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                      Constructions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; insert a triangle with isobarycenter and medians
(tm-define (make-fig-tri-gravity ls-len ls-lab mark)
  (let* ((z1 (list-ref (fig-tri-lengths->summits ls-len) 0))
  	 (z2 (list-ref (fig-tri-lengths->summits ls-len) 1))
  	 (z3 (list-ref (fig-tri-lengths->summits ls-len) 2))
  	 (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab))
  	 (m1 (fig-middle z2 z3))
  	 (m2 (fig-middle z1 z3))
  	 (m3 (fig-middle z2 z1)))	
  (make-fig-triangle ls-len ls-lab "" "" "" "")
  (make-fig-line `(,z1 ,m1))
  (make-fig-line `(,z2 ,m2))
  (make-fig-line `(,z3 ,m3))
  (make-fig-labels-poly `(,m1 ,m2 ,m3) ls-lab2 0.3)
  (if (string=? mark "codage")
      (begin
        (make-fig-mark-seg `(,z1 ,m3) "simple" 0.15 +i)
        (make-fig-mark-seg `(,m3 ,z2) "simple" 0.15 +i)
        (make-fig-mark-seg `(,z2 ,m1) "double" 0.15 +i)
        (make-fig-mark-seg `(,m1 ,z3) "double" 0.15 +i)
        (make-fig-mark-seg `(,z3 ,m2) "cross" 0.15 +i)
        (make-fig-mark-seg `(,m2 ,z1) "cross" 0.15 +i)))
  ))

;; insert a triangle with circumcircle
(tm-define (make-fig-tri-circumcircle ls-len ls-lab mark size)
  (let* ((z1 (list-ref (fig-tri-lengths->summits ls-len) 0))
  	 (z2 (list-ref (fig-tri-lengths->summits ls-len) 1))
  	 (z3 (list-ref (fig-tri-lengths->summits ls-len) 2))
  	 (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab))
  	 (m1 (fig-middle z2 z3))
  	 (m2 (fig-middle z1 z3))
  	 (m3 (fig-middle z2 z1)))	
  (make-fig-triangle ls-len ls-lab "" "" "" "")
  (make-fig-mediatrice `(,z2 ,z1) "" size size)
  (make-fig-mediatrice `(,z3 ,z2) "" size size)
  (make-fig-mediatrice `(,z1 ,z3) "" size size)
;  (make-fig-labels-poly `(,m1 ,m2 ,m3) ls-lab2 0.3)
  (graphics-set-mode "carc") 
  (insert `(carc ,@(fig-insert-pts `(,z1 ,z2 ,z3))))
  (make-fig-new-leave)
  (if (string=? mark "codage")
      (begin
        (make-fig-mark-seg `(,z1 ,m3) "simple" 0.15 +i)
        (make-fig-mark-seg `(,m3 ,z2) "simple" 0.15 +i)
        (make-fig-mark-seg `(,z2 ,m1) "double" 0.15 +i)
        (make-fig-mark-seg `(,m1 ,z3) "double" 0.15 +i)
        (make-fig-mark-seg `(,z3 ,m2) "cross" 0.15 +i)
        (make-fig-mark-seg `(,m2 ,z1) "cross" 0.15 +i)))
  ))

;; insert a triangle with orthocenter TODO : add the feet of the altitudes and the orthocenter
(tm-define (make-fig-tri-orthocenter ls-len ls-lab size)
  (let* ((z1 (list-ref (fig-tri-lengths->summits ls-len) 0))
  	 (z2 (list-ref (fig-tri-lengths->summits ls-len) 1))
  	 (z3 (list-ref (fig-tri-lengths->summits ls-len) 2))
  	 (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab)))	
  (make-fig-triangle ls-len ls-lab "" "" "" "")
  (make-fig-perpendicular `(,z2 ,z1) z3 "right" size size)
  (make-fig-perpendicular `(,z3 ,z2) z1 "right" size size)
  (make-fig-perpendicular `(,z1 ,z3) z2 "right" size size)
;  (make-fig-labels-poly `(,m1 ,m2 ,m3) ls-lab2 0.3)  ; define m1...
  ))
    
;; insert a triangle with bissectors and incircle
(tm-define (make-fig-tri-incircle ls-len ls-lab mark size)
  (let* ((z1 (list-ref (fig-tri-lengths->summits ls-len) 0))
  	 (z2 (list-ref (fig-tri-lengths->summits ls-len) 1))
  	 (z3 (list-ref (fig-tri-lengths->summits ls-len) 2))
  	 (z_center (fig-inter-lines z1 (fig-point-on-bisector z2 z1 z3 1) z2 (fig-point-on-bisector z3 z2 z1 1)))
  	 (z_pt (fig-projection z_center `(,z1 ,z2))) 
  	 (ls-lab2 (map (lambda (str) (string-append str "'")) ls-lab)))
  (make-fig-circle-pt z_center z_pt "none" "")
  (make-fig-triangle ls-len ls-lab "" "" "" "")
  (if (string=? mark "codage")
      (begin
        (make-fig-bisector `(,z2 ,z1 ,z3) "simple" 0.4 size size)
        (make-fig-bisector `(,z3 ,z2 ,z1) "double" 0.4 size size)
        (make-fig-bisector `(,z1 ,z3 ,z2) "triple" 0.4 size size)))
;  (make-fig-labels-poly `(,m1 ,m2 ,m3) ls-lab2 0.3)  ; define m1...
;  (graphics-set-mode "carc") 
;  (insert `(carc ,@(fig-insert-pts `(,z1 ,z2 ,z3))))
;  (make-fig-new-leave)
  )) 
  
;; graduated line                                   TODO
;; scale / unit=step of graduation / subdiv=number of subdivisions / ls-int='(min max)
(tm-define (make-grad-line scale ls-int unit subdiv labels)
  (let* ((z1 (list-ref ls-int 0))
         (z2 (list-ref ls-int 1)))
  (graphics-zoom (* 1.0 scale))
  (graphics-set-mode "line") 
  (insert `(with "line-arrows" (tuple (with "dash-style" "none" (line (tuple "-10ln" "6ln") (tuple "0ln" "0ln") (tuple "-10ln" "-6ln")))) (line ,@(fig-insert-pts ls-int))))
  (insert `(line ,@(fig-insert-pts ls-int)))
  (make-fig-new-leave)
  ))
  
  
